from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional, List
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_agents.genericAgentsReponse import InformationListeAgentsLA, InformationAgentLA
from wssiham.model.parametre.agent import (ParamAbsenceConges, ParamAffectationUO, ParamCorps, ParamFonction,
                                           ParamGrades, ParamGroupeHierarchique, ParamModaliteService, ParamPIP,
                                           ParamPositionsAdministratives, ParamQualitesStatutaires,
                                           ParamResponsHierarchiqUO, ParamSpecialitesNiv1Agent, ParamTypeContrat)
import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupListeAgents(Zeepclient):
    """ Le flux permet d’effectuer une recherche dans SIHAM selon plusieurs critères : recherche par nom,
    par affectation, …etc.
    """

    code_etablissement: str = field(default='')                 # code de l'établissement (exemple : 9740478B)
    date_observation: Optional[datetime] = field(default=None)
    nom_patronymique: str = field(default='')
    nom_usuel: str = field(default='')
    prenom: str = field(default='')
    numero_insee: str = field(default='')

    liste_absences_conges: List[ParamAbsenceConges] = field(default='')
    liste_affectation_uo: List[ParamAffectationUO] = field(default='')
    liste_corps: List[ParamCorps] = field(default='')
    liste_fonction: List[ParamFonction] = field(default='')
    liste_grade: List[ParamGrades] = field(default='')
    liste_groupe_hierarchique: List[ParamGroupeHierarchique] = field(default='')
    liste_modalite_service: List[ParamModaliteService] = field(default='')
    liste_pip: List[ParamPIP] = field(default='')
    liste_positions_administratives: List[ParamPositionsAdministratives] = field(default='')
    liste_qualites_statutaires: List[ParamQualitesStatutaires] = field(default='')
    liste_respons_hierarchiq_uo: List[ParamResponsHierarchiqUO] = field(default='')
    liste_type_contrat: List[ParamTypeContrat] = field(default='')
    specialites_niv1_agent: List[ParamSpecialitesNiv1Agent] = field(default='')

    tem_enseignant_chercheur: str = field(default='')                   # "1" = Enseignant/chercheur; "0" BIATSS

    # "A" = situation en vigueur à la date d'observation
    # "P" = situation en prévision par rapport à la date d'observation
    # "I" = situation clôturée par rapport à la date d'observation
    tem_etat: str = field(default='')

    temoin_valide: str = field(default='')

    date_fin_observation: Optional[datetime] = field(default=None)
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Optional[List[InformationAgentLA]]:
        """ Récupère les données administratives de l'agent

        Returns:
            Liste contenant les informations d'agents suivantes :
                - Code UAI de l’établissement
                - Matricule de l’agent
                - Numéro du dossier dans Harpege
                - Code de la qualité
                - Libellé de la civilité
                - Nom usuel
                - Prénom
                - Nom patronymique
                - Date de naissance
                - Numéro INSEE définitif
                - Numéro INSEE provisoire
                - Code poste de l’affectation hiérarchique dans l’établissement
                - Libellé long du code poste de l’affectation hiérarchique dans l’établissement
                - Code de l’emploi
                - Libellé long de l’emploi de l’affectation
                - Code de l’unité organisationnelle
                - Libellé long de l’unité organisationnelle
                - Date de début d’affectation hiérarchique
                - Date de fin d’affectation hiérarchique
                - Quotité d’affectation hiérarchique
                - Numéro de téléphone fixe professionnel
                - Numéro de portable professionnel
                - Numéro de fax professionnel
                - Adresse mail professionnelle
        """
        result = None
        try:
            paramrla = {
                'codeEtablissement': self.code_etablissement,
                'dateObservation': self.date_observation,
                'nomPatronymique': self.nom_patronymique,
                'nomUsuel': self.nom_usuel,
                'numeroInsee': self.numero_insee,
                'prenom': self.prenom,
                'temEnseignantChercheur': self.tem_enseignant_chercheur,
                'temEtat': self.tem_etat,
                'temoinValide': self.temoin_valide,
                'listeAbsencesConges': self.liste_absences_conges,
                'listeAffectationUO': self.liste_affectation_uo,
                'listeCorps': self.liste_corps,
                'listeFonction': self.liste_fonction,
                'listeGrades': self.liste_grade,
                'listeGroupeHierarchique': self.liste_groupe_hierarchique,
                'listeModaliteService': self.liste_modalite_service,
                'listePIP': self.liste_pip,
                'listePositionsAdministratives': self.liste_positions_administratives,
                'listeQualitesStatutaires': self.liste_qualites_statutaires,
                'listeResponsHierarchiqUO': self.liste_respons_hierarchiq_uo,
                'listeTypeContrat': self.liste_type_contrat,
                'specialitesniv1Agent': self.specialites_niv1_agent
            }

            resp = \
                self.zeep_client.service.recupListeAgents(ParamRecupListeAgents=paramrla)
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [InformationAgentLA(**ele) for ele in tmp_data]
                self.found = (True, len(result))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

