from dataclasses import dataclass, field
from typing import List, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.pec.genericPecResponse import GenericPecResponse
from wssiham.model.parametre.agent import *

import zeep
import logging
from datetime import datetime

logger = logging.getLogger(__name__)


@dataclass
class RenouvellementAgent(Zeepclient):
    """L’objectif de ce flux est de renouveler des agents."""
    matricule: str
    categorie_entree: str           # 'ACTIVE'
    code_administration: str        # 'RUN'
    code_etablissement: str         # '9740478B'
    date_renouvellement: datetime
    emploi: str = field(default='')
    temoin_validite: str = field(default='')

    liste_carriere: Optional[List[ParamCarriere]] = field(default=None)
    liste_contrats: Optional[List[ParamContrats]] = field(default=None)
    liste_modalites_services: Optional[List[ParamModalitesServicesPEC]] = field(default=None)
    liste_positions: Optional[List[ParamPostions]] = field(default=None)
    liste_statuts: Optional[List[ParamStatuts]] = field(default=None)
    motif_entree: str = field(default='')
    poste: str = field(default='')
    uo: str = field(default='')

    def response(self) -> Optional[GenericPecResponse]:
        """ L’appel à ce Web Service se fait par interrogation sur une liste de paramètres
        (état civil, date de naissance, affectations, carrière, statuts, position etc.).
        Afin de tracer dans le dossier agent l’information indiquant que le dossier ait été renouvelé via web-service,
        la méthode lors de la création crée une occurrence de Matricules précédents (ZYU5) avec les valeurs suivantes :
            -	IDEMPR (matricule précédent) : Valorisé par ‘WS_REN’
            -	DTCHNG (date de changement) : Valorisé par la date du jour

        Ce flux doit retourner une table :
        -	Tableau de résultat du renouvellement
        (statut du renouvellement, matricule de l’agent, anomalie rencontrée) : une occurrence par agent.

        Returns:
            Tableau de résultat du renouvellement
            (statut du renouvellement, matricule de l’agent, anomalie rencontrée) : une occurrence par agent.
        """
        result = {}
        try:
            resp = self.zeep_client.service.RenouvellementAgent(
                ParamRA={'categorieEntree': self.categorie_entree,
                         'codeAdministration': self.code_administration,
                         'codeEtablissement': self.code_etablissement,
                         'dateRenouvellement': self.date_renouvellement,
                         'emploi': self.emploi,
                         'listeCarriere': self.liste_carriere,
                         'listeContrats': self.liste_contrats,
                         'listeModalitesServices': self.liste_modalites_services,
                         'listePositions': self.liste_positions,
                         'listeStatuts': self.liste_statuts,
                         'matricule': self.matricule,
                         'motifEntree': self.motif_entree,
                         'poste': self.poste, 'temoinValidite': self.temoin_validite, 'UO': self.uo
                         }
            )
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = GenericPecResponse(**tmp_data)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result
